@@ -7,7 +7,7 @@
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-package Config::Model::Tk::ListEditor 1.372;
+package Config::Model::Tk::ListEditor 1.374;
 
 use strict;
 use warnings;
@@ -17,8 +17,8 @@ use Log::Log4perl;
 use base qw/Config::Model::Tk::ListViewer/;
 use subs qw/menu_struct/;
 use vars qw/$icon_path/;
-use Tk::Dialog;
 use Config::Model::Tk::NoteEditor;
+use Config::Model::Tk::CmeDialog;
 
 Construct Tk::Widget 'ConfigModelListEditor';
 
@@ -245,7 +245,7 @@ sub push_entry {
     eval { $list->fetch_with_id( scalar @idx ) };
 
     if ($@) {
-        $cw->Dialog(
+        $cw->CmeDialog(
             -title => "List index error",
             -text  => $@->as_string,
         )->Show;
@@ -350,7 +350,7 @@ sub try_and_redraw {
     eval { $to_try->(); };
 
     if ($@) {
-        $cw->Dialog(
+        $cw->CmeDialog(
             -title => "List index error",
             -text  => $@->as_string,
         )->Show;
